// jquery.visezjuste.toggleButton.js
//
// Creates a 'toggle' button with customized message and functionality.

$.fn.toggleButton = function(options){
	// Create options by merging options parameter with defaults.
	options = $.extend({}, $.fn.toggleButton.defaults, options);

	return this.each(function(){
			 var $t = $(this);			 
			 
			 // Note that in the code below, I've used the jQuery 'data' function. It's used to store
			 // information on a particular DOM element.
			 	
			 // The selector is the function or jQuery selector that 'selects' the object that the button will control.
			 // So, if options.selector is a function, get and store the result of the function. Otherwise, just store the value
			 // of options.selector.
			 $t.data('selector', $.isFunction(options.selector) ? options.selector.apply(this) : options.selector);


			 // The 'vals' are the button 'value's (i.e., the text that's displayed on the button face.)
			 // We expect to find a value for each toggle state, seperated by the 'value_split_str' from options.
			 // e.g., if 'value_split_str' is '|' the values from
			 // <input type="submit" class="ui-state-default ui-corner-all" id="toggle-pretest" value="Testez-vous!|Cachez le test" name="s" />
			 // would be 'Testez-vous!' and 'Cachez le test'.
			 $t.data('vals', $(this).attr('value').split(options.value_split_str));
			 
			 // This is an internal representation of the state of the button.
			 // Each time the button is clicked, we fire the next function, swap the button value, and increment the index (or set it to 0).
			 $t.data('index', 0);
	
	
			 this.update = function(index){
				 //From the 'functions' array, apply the function at 'index'.
				 options.functions[index].apply($t); 
				 
				 //Set the button 'value' to the string in 'vals' at 'index'.
				 $t.attr('value', $t.data('vals')[index]);		 
			 }	
	
	
			 // When we click the button...
			 $t.click(
			     function(){
			     // Grab a reference to $(this) as well be using it a few times.
				 var $t = $(this);
				 
				 // Grab the index out of "data".
				 var index = $t.data('index');

				 //Increment 'index' or loop back around if we're to the end.	
				 index = $t.data('vals').length <= index + 1 ? 0 : index + 1;
				 
				 //Stick index value back in data.
				 $t.data('index', index);
	
				 //Update based on new index.
				 this.update(index);
				 
			     }
			 );
	
			 //Start up the script by applying the specified init function.
			 options.init.apply(this);
		     });

};

$.fn.toggleButton.defaults = {
	//Note that this init function is for a button with 'show/hide' functionality. It hides the target element from the start.
    init : function(){$($(this).data('selector')).hide(); this.update(0);},
    
	//By default the selector for the DOM element we want to work on is the id of our button, minus 'toggle-', i.e.,
	//if our button id is 'toggle-test' the id of the element it will work on is 'test'
	selector : function(){return '#' + $(this).attr('id').split('toggle-').join('')},
    value_split_str : '|',
	//The functions we're going to call on our target, each time the button is clicked.
	//Only one function is called per click. Functions are called in order. If we reach the end of the array, we loop.
	//There should be one function per value in the button 'value'.
	functions : [function(){$($(this).data('selector')).slideUp()}, function(){$($(this).data('selector')).slideDown()}]
};
